<?php
declare(strict_types = 1);

/**
 * Migration d'iGalerie 2 vers iGalerie 3.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
if (!is_dir(__DIR__ . '/includes/core/')
 || !is_file(__DIR__ . '/capture.php'))
{
	die('iGalerie 3 non détecté.');
}
require_once(__DIR__ . '/includes/prepend.php');

Template::init();
Migration::init();
Template::start();
$tpl =& Template::$data;



class Migration
{
	/**
	 * Initialisation.
	 *
	 * @return void
	 */
	public static function init(): void
	{	
		if (empty($_POST['upgrade']))
		{
			return;
		}

		// On repousse les limites.
		if (function_exists('ini_set'))
		{
			ini_set('memory_limit', '2048M');
		}
		if (function_exists('set_time_limit'))
		{
			set_time_limit(3600);
		}

		$params = self::_getConfigV2();
		if (!$params)
		{
			Template::set('error', 'Erreur du fichier de configuration.');
			return;
		}

		// Connexion à la base de données MySQL de la version 2.
		if (!DBV2::connect($params['host'], $params['port'], $params['name'],
			$params['user'], $params['pass'], $params['pref']))
		{
			Template::set('error', DBV2::getError());
			return;
		}

		// Récupération des enregistrements de la table {config}.
		if (!DBV2::execute("SELECT * FROM {config}"))
		{
			Template::set('error', DBV2::getError());
			return;
		}
		$config_v2 = DBV2::fetchAll('conf_name', 'conf_value');

		// Récupération des enregistrements des tables de la version 2.
		$tables = ['cameras_brands', 'cameras_models', 'cameras_models_images',
			'categories', 'comments', 'favorites', 'groups', 'groups_perms',
			'images', 'tags', 'tags_images', 'users', 'votes'];
		foreach ($tables as $table)
		{
			$order_by = "";
			if ($table == 'categories')
			{
				$order_by = "ORDER BY cat_path ASC";
			}
			if (!DBV2::execute("SELECT * FROM {{$table}} $order_by"))
			{
				Template::set('error', DBV2::getError());
				return;
			}
			${$table . '_v2'} = DBV2::fetchAll();
		}

		// Déconnexion de la base de données de la version 2.
		DBV2::disconnect();

		// Connexion à la base de données de la version 3.
		if (!DB::connect())
		{
			Template::set('error', DB::getError());
			return;
		}
		if (DBLayer::NAME == 'PostgreSQL')
		{
			Template::set('warning', 'Impossible d\'effectuer la migration vers PostgreSQL');
			return;
		}

		// On démarre une transaction.
		if (!DB::beginTransaction())
		{
			Template::set('error', DB::getError());
			return;
		}

		// Migration déjà effectuée ?
		$sql = "SELECT * FROM {config} WHERE conf_name = 'migration_igalerie2'";
		if (!DB::execute($sql))
		{
			Template::set('error', DB::getError());
			return;
		}
		if (DB::fetchVal())
		{
			Template::set('warning', 'La migration a déjà été effectuée.');
			return;
		}

		// Mise à jour des tables d'iGalerie 3.
		if (!self::_updateGroups($groups_v2)
		 || !self::_updateUsers($users_v2)
		 || !self::_updateCategories($categories_v2)
		 || !self::_updateGroupsPermissions($groups_perms_v2)
		 || !self::_updateItems($images_v2)
		 || !self::_updateComments($comments_v2)
		 || !self::_updateVotes($votes_v2)
		 || !self::_updateFavorites($favorites_v2)
		 || !self::_updateTags($tags_v2)
		 || !self::_updateTagsItems($tags_images_v2)
		 || !self::_updateCamerasBrands($cameras_brands_v2)
		 || !self::_updateCamerasModels($cameras_models_v2)
		 || !self::_updateCamerasItems($cameras_models_images_v2)
		 || !self::_updateConfig($config_v2))
		{
			DB::rollbackTransaction();
			return;
		}

		// Changement des noms de fichier si nécessaire.
		$sql = "SELECT item_id, item_path FROM {items} WHERE item_path LIKE '%.%.%'";
		if (!DB::execute($sql))
		{
			Template::set('error', DB::getError());
			return;
		}
		$items_rename = [];
		foreach (DB::fetchAll('item_id', 'item_path') as $id => &$path)
		{
			$f = App::getValidFilename(basename($path));
			if ($f !== basename($path))
			{
				$items_rename[$id] = ['item_path' => $f];
			}
		}
		if ($items_rename && Item::edit($items_rename) < 0)
		{
			Template::set('error', 'Impossible de renommer les fichiers.');
			return;
		}

		// Vignettes externes des catégories.
		$sql = "SELECT cat_id, cat_path FROM {categories} WHERE thumb_id = 0 AND cat_id > 1";
		if (DB::execute($sql))
		{
			$categories = DB::fetchAll();
			$params = [];
			foreach ($categories as &$i)
			{
				$sql = "SELECT item_id
						  FROM {items}
						 WHERE item_path LIKE ?
					  ORDER BY item_adddt DESC
					     LIMIT 1";
				if (DB::execute($sql, DB::likeEscape($i['cat_path']) . '/%'))
				{
					$item_id = (int) DB::fetchVal();
					if ($item_id)
					{
						$params[] = [$item_id, $i['cat_id']];
					}
				}
			}
			if ($params)
			{
				$sql = "UPDATE {categories} SET thumb_id = ? WHERE cat_id = ?";
				DB::execute($sql, $params);
			}
		}

		// Vérification et correction des statistiques.
		Maintenance::dbStats();

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			Template::set('error', DB::getError());
			return;
		}

		Template::set('success', 'Migration effectuée avec succès !');
	}



	/**
	 * Vérifie et modifie une date si nécessaire.
	 *
	 * @param string $str
	 *
	 * @return string
	 */
	private static function _changeDate(string &$date): string
	{
		if ($date === '0000-00-00 00:00:00' || $date === '')
		{
			$date = '2020-01-01 00:00:00';
		}
		if ($date === '0000-00-00')
		{
			$date = '2020-01-01';
		}

		return $date;
	}

	/**
	 * Récupération des paramètres de base de données de la version 2.
	 *
	 * @return array
	 */
	private static function _getConfigV2(): array
	{
		$f = GALLERY_ROOT . '/config/confv2.php';

		if (!file_exists($f))
		{
			return [];
		}

		$params = [];

		$content = file($f);
		foreach ($content as &$line)
		{
			if (strstr($line, 'CONF_DB_'))
			{
				$regex = '`\WCONF_DB_([A-Z]{3,4})[\'"]\s*,\s*[\'"]([^\'"]*)[\'"]`';
				if (preg_match($regex, $line, $m))
				{
					switch ($m[1])
					{
						case 'DSN' :
							$regex = '`mysql:host=([^;]+);(?:port=([^;]+);)?dbname=([^\'"]+)`';
							if (preg_match($regex, $line, $m))
							{
								$params['host'] = $m[1];
								$params['port'] = $m[2] ? $m[2] : 3306;
								$params['name'] = $m[3];
							}
							break;

						case 'PASS' :
							$params['pass'] = $m[2];
							break;

						case 'PREF' :
							$params['pref'] = $m[2];
							break;

						case 'USER' :
							$params['user'] = $m[2];
							break;
					}
				}
			}
		}

		return (empty($params['user']) || empty($params['host'])) ? [] : $params;
	}

	/**
	 * Insertion de lignes dans une table.
	 *
	 * @param string $table
	 * @param array $params
	 *
	 * @return bool
	 */
	private static function _insert(string $table, array $params): bool
	{
		// Mise à jour de la base de données.
		$values = substr(str_repeat('?, ', count($params[0])), 0, -2);
		$columns = implode(', ', array_keys($params[0]));
		$sql = "INSERT IGNORE INTO {{$table}} ($columns) VALUES ($values)";
		foreach ($params as &$cols)
		{
			$cols = array_values($cols);
		}

		if (!DB::execute($sql, $params))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Remplace le code des smilies par des émojis.
	 *
	 * @param string $str
	 *
	 * @return string
	 */
	private static function _smiliesToEmojis(string $str): string
	{
		$str = preg_replace('`:\?\?\?:`', "\u{1F615}", $str);
		$str = preg_replace('`:cool:`', "\u{1F60E}", $str);
		$str = preg_replace('`:cry:`', "\u{1F622}", $str);
		$str = preg_replace('`:eek:`', "\u{1F62E}", $str);
		$str = preg_replace('`:evil:`', "\u{1F608}", $str);
		$str = preg_replace('`:grin:`', "\u{1F600}", $str);
		$str = preg_replace('`:lol:`', "\u{1F923}", $str);
		$str = preg_replace('`:mad:`', "\u{1F620}", $str);
		$str = preg_replace('`:neutral:`', "\u{1F610}", $str);
		$str = preg_replace('`:oops:`', "\u{1F633}", $str);
		$str = preg_replace('`:razz:`', "\u{1F61B}", $str);
		$str = preg_replace('`:roll:`', "\u{1F644}", $str);
		$str = preg_replace('`:sad:`', "\u{1F641}", $str);
		$str = preg_replace('`:smile:`', "\u{1F603}", $str);
		$str = preg_replace('`:surprise:`', "\u{1F632}", $str);
		$str = preg_replace('`:twisted:`', "\u{1F608}", $str);
		$str = preg_replace('`:wink:`', "\u{1F609}", $str);
		$str = preg_replace('`:yell:`', "\u{1F606}", $str);

		return $str;
	}

	/**
	 * Insertion de lignes dans une table.
	 *
	 * @param string $table
	 * @param array $params
	 *
	 * @return bool
	 */
	private static function _update(string $table, array $params, string $col_where): bool
	{
		// Mise à jour des groupes spéciaux.
		$columns = '';
		foreach (array_keys($params[0]) as &$col)
		{
			if ($col != $col_where)
			{
				$columns .= $col . ' = :' . $col . ', ';
			}
		}
		$columns = substr($columns, 0, -2);
		$sql = "UPDATE {{$table}} SET $columns WHERE $col_where = :$col_where";

		if (!DB::execute($sql, $params))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Supprime les textes localisés de la version 2.
	 *
	 * @param mixed $str
	 *
	 * @return mixed
	 */
	private static function _removeLocale(&$str)
	{
		if (is_string($str))
		{
			$replace = preg_replace('`<[a-z]{2}_[A-Z]{2}>[^$]*$`', '', $str);
			if ($replace !== '')
			{
				return $replace;
			}
		}

		return $str;
	}

	/**
	 * Garanti l'unicité de la valeur $value dans le tableau $unique.
	 *
	 * @param array $unique
	 * @param value $value
	 *
	 * @return void
	 */
	private static function _unique(&$unique, &$value)
	{
		$n = 2;
		$test = $value;
		while (in_array($test, $unique))
		{
			$test = $value . '-' . $n;
			$n++;
		}
		$value = $test;
		if (!in_array($value, $unique))
		{
			$unique[] = $value;
		}
	}

	/**
	 * Mise à jour de la table {cameras_brands}.
	 *
	 * @param array $cameras_brands_v2
	 *
	 * @return bool
	 */
	private static function _updateCamerasBrands(array &$cameras_brands_v2): bool
	{
		if (!$cameras_brands_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {cameras_brands}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal() > 1)
		{
			Template::set('warning', 'La version 3 contient déjà'
				. ' des marques d\'appareils photos.');
			return FALSE;
		}

		// On garanti l'unicité de camera_brand_url et camera_brand_name.
		$url = [];
		$name = [];
		foreach ($cameras_brands_v2 as &$i)
		{
			if (!$i['camera_brand_url'])
			{
				$i['camera_brand_url'] = '-';
			}
			if (!$i['camera_brand_name'])
			{
				$i['camera_brand_name'] = '?';
			}

			self::_unique($url, $i['camera_brand_url']);
			self::_unique($name, $i['camera_brand_name']);
		}

		// Mise à jour de la base de données.
		if (!self::_insert('cameras_brands', $cameras_brands_v2))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {cameras_items}.
	 *
	 * @param array $cameras_models_images_v2
	 *
	 * @return bool
	 */
	private static function _updateCamerasItems(array &$cameras_models_images_v2): bool
	{
		if (!$cameras_models_images_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {cameras_items}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal() > 1)
		{
			Template::set('warning', 'La version 3 contient déjà'
				. ' des modèles d\'appareils photos.');
			return FALSE;
		}

		$cameras_items = [];

		foreach ($cameras_models_images_v2 as &$i)
		{
			$cameras_items[] =
			[
				'camera_model_id' => $i['camera_model_id'],
				'item_id' => $i['image_id']
			];
		}

		// Mise à jour de la base de données.
		if ($cameras_items && !self::_insert('cameras_items', $cameras_items))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {cameras_models}.
	 *
	 * @param array $cameras_models_v2
	 *
	 * @return bool
	 */
	private static function _updateCamerasModels(array &$cameras_models_v2): bool
	{
		if (!$cameras_models_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {cameras_models}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal() > 1)
		{
			Template::set('warning', 'La version 3 contient déjà'
				. ' des modèles d\'appareils photos.');
			return FALSE;
		}

		// On garanti l'unicité de camera_model_url et camera_model_name.
		$url = [];
		$name = [];
		foreach ($cameras_models_v2 as &$i)
		{
			if (!$i['camera_model_url'])
			{
				$i['camera_model_url'] = '-';
			}
			if (!$i['camera_model_name'])
			{
				$i['camera_model_name'] = '?';
			}

			self::_unique($url, $i['camera_model_url']);
			self::_unique($name, $i['camera_model_name']);
		}

		// Mise à jour de la base de données.
		if (!self::_insert('cameras_models', $cameras_models_v2))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {categories}.
	 *
	 * @param array $categories_v2
	 *
	 * @return bool
	 */
	private static function _updateCategories(array &$categories_v2): bool
	{
		if (!$categories_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {categories}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal() > 1)
		{
			Template::set('warning', 'La version 3 contient déjà des catégories.');
			return FALSE;
		}

		$cat_one = [];
		foreach ($categories_v2 as $k => &$i)
		{
			$i['cat_a_rating'] = $i['cat_a_rate'];
			$i['cat_d_rating'] = $i['cat_d_rate'];
			$i['cat_orderby'] = $i['cat_orderby'] === NULL
				? NULL
				: str_replace('image', 'item', $i['cat_orderby']);
			$i['cat_parents'] = str_replace(':', Parents::SEPARATOR, $i['cat_parents']);
			$i['cat_lastpubdt'] = $i['cat_lastadddt'];

			unset($i['cat_a_rate']);
			unset($i['cat_d_rate']);
			unset($i['cat_password']);
			unset($i['cat_lastadddt']);
			unset($i['cat_place']);
			unset($i['cat_tb_infos']);
			unset($i['cat_style']);
			unset($i['cat_watermark']);

			if ($i['cat_id'] == 1)
			{
				unset($i['user_id']);
				unset($i['thumb_id']);
				unset($i['parent_id']);
				unset($i['cat_parents']);
				unset($i['cat_path']);
				unset($i['cat_url']);
				unset($i['cat_name']);
				unset($i['cat_lat']);
				unset($i['cat_long']);
				unset($i['cat_votable']);
				unset($i['cat_commentable']);
				unset($i['cat_creatable']);
				unset($i['cat_uploadable']);
				unset($i['cat_downloadable']);
				unset($i['cat_status']);
				unset($i['cat_position']);
				unset($i['cat_filemtime']);
				$cat_one = $i;
				unset($categories_v2[$k]);
			}
		}

		// Mise à jour de la catégorie 1.
		if (!self::_update('categories', [$cat_one], 'cat_id'))
		{
			return FALSE;
		}

		// Insertion des catégories.
		if ($categories_v2 && !self::_insert('categories', array_values($categories_v2)))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {comments}.
	 *
	 * @param array $comments_v2
	 *
	 * @return bool
	 */
	private static function _updateComments(array &$comments_v2): bool
	{
		// Aucun commentaire.
		if (!$comments_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {comments}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal())
		{
			Template::set('warning', 'La version 3 contient déjà des commentaires.');
			return FALSE;
		}

		// Modification des colonnes.
		foreach ($comments_v2 as &$cols)
		{
			$cols['com_message'] = self::_smiliesToEmojis((string) $cols['com_message']);
			Utility::arrayKeyRename('image_id', 'item_id', $cols);
		}

		return self::_insert('comments', $comments_v2);
	}

	/**
	 * Mise à jour de la table {config}.
	 *
	 * @param array $config_v2
	 *
	 * @return bool
	 */
	private static function _updateConfig(array &$config_v2): bool
	{
		// Récupération de la configuration de la v3.
		Config::getDBParams();
		if (!$config_v3 = Config::$params)
		{
			Template::set('warning', 'Configuration manquante');
			return FALSE;
		}

		$set_value = function(string $conf_name_v2, string $conf_name_v3 = '')
		use (&$config_v2, &$config_v3): void
		{
			$conf_name_v3 = $conf_name_v3 === '' ? $conf_name_v2 : $conf_name_v3;
			if (!isset($config_v3[$conf_name_v3]))
			{
				l("Paramètre $conf_name_v3 inconnu.");
				return;
			}
			$config_v3[$conf_name_v3] = $config_v2[$conf_name_v2] ?? $config_v3[$conf_name_v3];
		};

		// Listes noires.
		$set_value('blacklist_emails');
		$set_value('blacklist_ips');
		$set_value('blacklist_names');
		$set_value('blacklist_words');

		// Commentaires.
		$set_value('comments');
		$set_value('comments_antiflood');
		$set_value('comments_smilies', 'comments_emoji_picker');
		$set_value('comments_maxchars');
		$set_value('comments_maxlines');
		$set_value('comments_maxurls');
		$set_value('comments_moderate');
		$set_value('comments_order', 'comments_order_by');
		$set_value('comments_required_email');
		$set_value('comments_required_website');
		$set_value('comments_convert_urls', 'comments_url_to_link');
		$set_value('comments_links_maxlength', 'comments_url_to_link_maxlength');

		// Informations EXIF.
		$set_value('exif');

		// Diaporama.
		$set_value('diaporama');
		$set_value('diaporama_auto_loop');
		$set_value('diaporama_auto_start');
		$set_value('diaporama_carousel');

		// Paramètres de la galerie.
		$set_value('gallery_closure', 'gallery_closed');
		$set_value('gallery_closure_message', 'gallery_closed_text');
		$set_value('gallery_description');
		$set_value('gallery_footer_message', 'gallery_footer_text');
		$set_value('gallery_title');

		// Géolocalisation.
		$set_value('geoloc', 'geolocation');

		// Informations IPTC et XMP.
		$set_value('iptc');
		$config_v3['iptc_get_data'] = !empty($config_v2['iptc_description'])
			|| !empty($config_v2['iptc_keywords'])
			|| !empty($config_v2['iptc_title']);
		$set_value('xmp');

		// RSS.
		$set_value('rss');
		$set_value('rss_max_items');

		// Fichiers.
		$set_value('recent_images', 'items_recent');
		$set_value('recent_images_time', 'items_recent_days');
		if ($config_v2['images_resize_method'] == 2)
		{
			$set_value('images_resize', 'items_resize');
		}
		$set_value('images_resize_gd_height', 'items_resize_height');
		$set_value('images_resize_gd_width', 'items_resize_width');
		$set_value('images_resize_gd_quality', 'items_resize_quality');

		// Liens externes.
		if (is_array($widgets_params = unserialize($config_v2['widgets_params'] ?? '')))
		{
			$links = $widgets_params['links'];
			if (is_array($links['items']))
			{
				foreach ($links['items'] as &$params)
				{
					Utility::arrayKeyRename('activate', 'activated', $params);
				}
				$config_v3['links'] = !empty($links['status']);
				$config_v3['links_params'] = $links['items'];
			}
		}

		// Courriel : paramètres.
		$set_value('mail_auto_bcc');
		$set_value('mail_auto_primary_recipient_address');
		$set_value('mail_auto_sender_address');

		// Pages.
		$pages_order_v2 = unserialize($config_v2['pages_order'] ?? '');
		$pages_order_v3 = Utility::jsonDecode($config_v3['pages_order']);
		if (is_array($pages_order_v2) && is_array($pages_order_v3))
		{
			$pages_order = [];
			foreach ($pages_order_v2 as $k => &$page)
			{
				if (substr($page, 0, 6) == 'perso_')
				{
					$pages_order[] = str_replace('perso', 'custom', $page);
				}
				else if (in_array($page, Config::PAGES_ORDER))
				{
					$pages_order[] = $page;
				}
			}
			$config_v3['pages_order'] = $pages_order;
		}
		$pages_params_v2 = unserialize($config_v2['pages_params'] ?? '');
		$pages_params_v3 = Utility::jsonDecode($config_v3['pages_params']);
		if (is_array($pages_params_v2) && is_array($pages_params_v3))
		{
			$pages_params = [];
			foreach ($pages_params_v2 as $page => &$params)
			{
				if (substr($page, 0, 6) == 'perso_')
				{
					if (isset($params['file']))
					{
						Utility::arrayKeyRename('file', 'filename', $params);
					}
					else
					{
						$params['filename'] = '';
					}
					if (isset($params['text']))
					{
						Utility::arrayKeyRename('text', 'content', $params);
					}
					else
					{
						$params['content'] = '';
					}
					if (isset($params['type']))
					{
						unset($params['type']);
					}
					$pages_params[str_replace('perso', 'custom', $page)] = $params;
				}
				else if (in_array($page, Config::PAGES_ORDER))
				{
					$pages_params[$page] = $params;
				}
			}
			$config_v3['pages_params'] = $pages_params;
		}

		// Moteur de recherche.
		$set_value('search');
		$set_value('search_advanced');

		// Tags.
		$set_value('tags');

		// Envoi de fichiers.
		$set_value('upload_resize');
		$set_value('upload_resize_maxwidth');
		$set_value('upload_resize_maxheight');
		$set_value('upload_resize_quality');

		// Utilisateurs.
		$set_value('users');
		$set_value('users_inscription', 'users_registration');
		$set_value('users_inscription_password', 'users_registration_password');
		$set_value('users_inscription_password_text', 'users_registration_password_text');
		$set_value('users_inscription_moderate', 'users_registration_valid_admin');
		$set_value('users_inscription_by_mail', 'users_registration_valid_email');
		$set_value('users_inscription_by_password', 'users_registration_valid_password');
		$set_value('users_only_members');

		// Paramètres de profil utilisateur.
		$profile_params_v2 = unserialize($config_v2['users_profile_infos'] ?? '');
		$profile_params_v3 = Utility::jsonDecode($config_v3['users_profile_params']);
		if (is_array($profile_params_v2) && is_array($profile_params_v3))
		{
			if (isset($profile_params_v2['infos']))
			{
				$list =
				[
					'birthdate' => 'birthdate',
					'description' => 'desc',
					'email' => 'email',
					'firstname' => 'firstname',
					'gender' => 'sex',
					'location' => 'loc',
					'name' => 'name',
					'website' => 'website'
				];
				foreach ($list as $v3 => $v2)
				{
					$profile_params_v3[$v3] =
					[
						'activated' => $profile_params_v2['infos'][$v2]['activate'],
						'required' => $profile_params_v2['infos'][$v2]['required']
					];
				}
				for ($i = 1; $i <= 5; $i++)
				{
					if (!isset($profile_params_v2['perso']['perso_' . $i]))
					{
						continue;
					}
					$profile_params_v3['custom_' . $i] =
					[
						'activated' => $profile_params_v2['perso']['perso_' . $i]['activate'],
						'required' => $profile_params_v2['perso']['perso_' . $i]['required'],
						'name' => $profile_params_v2['perso']['perso_' . $i]['name']
					];
				}
			}
			$config_v3['users_profile_params'] = $profile_params_v3;
		}

		// Vues.
		$set_value('nohits_useragent', 'views_useragent');
		$set_value('nohits_useragent_list', 'views_useragent_list');

		// Votes.
		$set_value('votes');

		// Mise à jour de la base de données.
		if (Config::changeDBParams($config_v3) === -1)
		{
			return FALSE;
		}
		$sql = "INSERT INTO {config} (conf_name, conf_value)"
			 . " VALUES ('migration_igalerie2', '1')";
		if (!DB::execute($sql))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {favorites}.
	 *
	 * @param array $cameras_models_images_v2
	 *
	 * @return bool
	 */
	private static function _updateFavorites(array &$favorites_v2): bool
	{
		if (!$favorites_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {favorites}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal() > 1)
		{
			Template::set('warning', 'La version 3 contient déjà des favoris.');
			return FALSE;
		}

		$favorites = [];

		foreach ($favorites_v2 as &$i)
		{
			$favorites[] =
			[
				'item_id' => $i['image_id'],
				'user_id' => $i['user_id'],
				'fav_date' => self::_changeDate($i['fav_date'])
			];
		}

		// Mise à jour de la base de données.
		if ($favorites && !self::_insert('favorites', $favorites))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {groups}.
	 *
	 * @param array $groups_v2
	 *
	 * @return bool
	 */
	private static function _updateGroups(array &$groups_v2):bool
	{
		if (!DB::execute("SELECT COUNT(*) FROM {groups}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal() > 3)
		{
			Template::set('warning', 'La version 3 contient déjà des groupes.');
			return FALSE;
		}

		$groups_s = [];
		$groups_u = [];
		foreach ($groups_v2 as &$cols)
		{
			// Textes.
			self::_removeLocale($cols['group_name']);
			self::_removeLocale($cols['group_title']);
			self::_removeLocale($cols['group_desc']);

			// Date.
			self::_changeDate($cols['group_crtdt']);

			// Permissions d'accès aux fonctionnalités.
			$perms_v2 = unserialize($cols['group_perms'] ?? '');
			$perms_v3 = Config::USERS_GROUP_PERMS_DEFAULT;
			if (is_array($perms_v2))
			{
				$perms_v3['comments_add']
					= $perms_v2['gallery']['perms']['add_comments']
					?? $perms_v3['comments_add'];
				$perms_v3['comments_add_mode']
					= $perms_v2['gallery']['perms']['add_comments_mode']
					?? $perms_v3['comments_add_mode'];
				$perms_v3['comments_read']
					= $perms_v2['gallery']['perms']['read_comments']
					?? $perms_v3['comments_read'];
				$perms_v3['image_original']
					= $perms_v2['gallery']['perms']['image_original']
					?? $perms_v3['image_original'];
				$perms_v3['members_profile']
					= $perms_v2['gallery']['perms']['members_list']
					?? $perms_v3['members_profile'];
				$perms_v3['perm_list']
					= $perms_v2['gallery']['perms']['access_list']
					?? $perms_v3['perm_list'];
				$perms_v3['upload']
					= $perms_v2['gallery']['perms']['upload']
					?? $perms_v3['upload'];
				$perms_v3['upload_mode']
					= $perms_v2['gallery']['perms']['upload_mode']
					?? $perms_v3['upload_mode'];
				$perms_v3['upload_owner']
					= $perms_v2['gallery']['perms']['upload_create_owner']
					?? $perms_v3['upload_owner'];
				$perms_v3['votes']
					= $perms_v2['gallery']['perms']['votes']
					?? $perms_v3['votes'];
			}
			$cols['group_perms'] = Utility::jsonEncode($perms_v3);

			if ($cols['group_id'] < 4)
			{
				unset($cols['group_crtdt']);
				$groups_s[] = $cols;
			}
			else
			{
				$groups_u[] = $cols;
			}
		}

		// Mise à jour des groupes spéciaux.
		if ($groups_s && !self::_update('groups', $groups_s, 'group_id'))
		{
			return FALSE;
		}

		// Création des groupes utilisateur.
		if ($groups_u && !self::_insert('groups', $groups_u))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {groups_permissions}.
	 *
	 * @param array $groups_perms_v2
	 *
	 * @return bool
	 */
	private static function _updateGroupsPermissions(array &$groups_perms_v2): bool
	{
		if ($groups_perms_v2)
		{
			$sql = "INSERT INTO {groups_permissions} (group_id, cat_id, perm_list)
				VALUES (:group_id, :cat_id, :perm_list)";
			if (!DB::execute($sql, $groups_perms_v2))
			{
				return FALSE;
			}
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {items}.
	 *
	 * @param array $images_v2
	 *
	 * @return bool
	 */
	private static function _updateItems(array &$images_v2): bool
	{
		if (!$images_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {items}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal())
		{
			Template::set('warning', 'La version 3 contient déjà des fichiers.');
			return FALSE;
		}

		$items = [];
		foreach ($images_v2 as &$i)
		{
			// Type de fichier.
			$file = CONF_ALBUMS_PATH . '/' . $i['image_path'];
			$type = 0;
			if (file_exists($file))
			{
				$type = Item::getTypeCode(CONF_ALBUMS_PATH . '/' . $i['image_path']);
			}
			$type = $type === 0 ? Item::TYPE_JPEG : $type;

			$items[] =
			[
				'item_id' => $i['image_id'],
				'user_id' => $i['user_id'],
				'cat_id' => $i['cat_id'],
				'item_type' => $type,
				'item_path' => $i['image_path'],
				'item_url' => $i['image_url'],
				'item_width' => $i['image_width'],
				'item_height' => $i['image_height'],
				'item_filesize' => $i['image_filesize'],
				'item_exif' => $i['image_exif'],
				'item_iptc' => $i['image_iptc'],
				'item_xmp' => $i['image_xmp'],
				'item_orientation' => $i['image_rotation'],
				'item_lat' => $i['image_lat'],
				'item_long' => $i['image_long'],
				'item_name' => $i['image_name'],
				'item_desc' => $i['image_desc'],
				'item_adddt' => $i['image_adddt'],
				'item_pubdt' => $i['image_adddt'],
				'item_crtdt' => $i['image_crtdt'],
				'item_hits' => $i['image_hits'],
				'item_comments' => $i['image_comments'],
				'item_votes' => $i['image_votes'],
				'item_rating' => $i['image_rate'],
				'item_status' => $i['image_status'],
				'item_position' => $i['image_position']
			];
		}

		// Insertion des fichiers.
		if ($items && !self::_insert('items', $items))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {tags}.
	 *
	 * @param array $tags_v2
	 *
	 * @return bool
	 */
	private static function _updateTags(array &$tags_v2): bool
	{
		if (!$tags_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {tags}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal())
		{
			Template::set('warning', 'La version 3 contient déjà des tags.');
			return FALSE;
		}

		// On garanti l'unicité de tag_url et tag_name.
		$url = [];
		$name = [];
		foreach ($tags_v2 as &$i)
		{
			if (!$i['tag_url'])
			{
				$i['tag_url'] = '-';
			}
			if (!$i['tag_name'])
			{
				$i['tag_name'] = '?';
			}

			self::_unique($url, $i['tag_url']);
			self::_unique($name, $i['tag_name']);
		}

		// Insertion des tags.
		if (!self::_insert('tags', $tags_v2))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {tags_items}.
	 *
	 * @param array $tags_images_v2
	 *
	 * @return bool
	 */
	private static function _updateTagsItems(array &$tags_images_v2): bool
	{
		if (!$tags_images_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {tags_items}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal())
		{
			Template::set('warning', 'La version 3 contient déjà des tags.');
			return FALSE;
		}

		$tags_items = [];
		foreach ($tags_images_v2 as &$i)
		{
			if (isset($i['tag_id']))
			{
				$tags_items[] =
				[
					'tag_id' => $i['tag_id'],
					'item_id' => $i['image_id']
				];
			}
		}

		// Insertion des associations tags - fichiers.
		if ($tags_items && !self::_insert('tags_items', $tags_items))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Mise à jour de la table {users}.
	 *
	 * @param array $users_v2
	 *
	 * @return bool
	 */
	private static function _updateUsers(array &$users_v2): bool
	{
		if (!DB::execute("SELECT COUNT(*) FROM {users}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal() > 2)
		{
			Template::set('warning', 'La version 3 contient déjà des utilisateurs.');
			return FALSE;
		}

		$super_admin = [];
		$users = [];
		foreach ($users_v2 as &$cols)
		{
			if ($cols['user_id'] == 2)
			{
				continue;
			}

			$params =
			[
				'user_id' => $cols['user_id'],
				'group_id' => $cols['group_id'],
				'user_login' => $cols['user_login'],
				'user_password' => Utility::passwordHash(Utility::key()),
				'user_name' => $cols['user_name'],
				'user_firstname' => $cols['user_firstname'],
				'user_gender' => $cols['user_sex'] ? $cols['user_sex'] : '?',
				'user_birthdate' => $cols['user_birthdate'],
				'user_location' => $cols['user_loc'],
				'user_description' => $cols['user_desc'],
				'user_email' => $cols['user_email'],
				'user_website' => $cols['user_website'],
				'user_avatar' => $cols['user_avatar'],
				'user_alert' => $cols['user_alert'],
				'user_lang' => 'fr',
				'user_tz' => $cols['user_tz'],
				'user_status' => $cols['user_status'],
				'user_crtdt' => $cols['user_crtdt'],
				'user_crtip' => $cols['user_crtip'],
				'user_lastvstdt' => $cols['user_lastvstdt'],
				'user_lastvstip' => $cols['user_lastvstip'],
				'user_custom_1' => NULL,
				'user_custom_2' => NULL,
				'user_custom_3' => NULL,
				'user_custom_4' => NULL,
				'user_custom_5' => NULL
			];

			// Champs personnalisés.
			$user_other = unserialize($cols['user_other'] ?? '');
			if (is_array($user_other))
			{
				for ($i = 1; $i <= 5; $i++)
				{
					if (!empty($user_other['perso_' . $i]))
					{
						$params['user_custom_' . $i] = $user_other['perso_' . $i];
					}
				}
			}

			// Avatar.
			if ($cols['user_avatar'])
			{
				$avatar_file_v2 = __DIR__ . '/users/avatars/user'
					. $cols['user_id'] . '.jpg';
				$avatar_file_v2_thumb = __DIR__ . '/users/avatars/user'
					. $cols['user_id'] . '_thumb.jpg';
				if (file_exists($avatar_file_v2) && file_exists($avatar_file_v2_thumb))
				{
					$avatar_file_v3 = __DIR__ . '/images/avatars/'
						. Avatar::getFilename((int) $cols['user_id'], FALSE);
					$avatar_file_v3_thumb = __DIR__ . '/images/avatars/'
						. Avatar::getFilename((int) $cols['user_id'], TRUE);
					File::copy($avatar_file_v2, $avatar_file_v3);
					File::copy($avatar_file_v2, $avatar_file_v3_thumb);
				}
			}

			if ($cols['user_id'] == 1)
			{
				unset($params['user_login']);
				unset($params['user_password']);
				unset($params['user_lang']);
				unset($params['user_tz']);
				unset($params['user_status']);
				$super_admin = $params;
			}
			else
			{
				$users[] = $params;
			}
		}

		// Mise à jour des informations du super-admin.
		if (!self::_update('users', [$super_admin], 'user_id'))
		{
			return FALSE;
		}

		if (!$users)
		{
			return TRUE;
		}

		return self::_insert('users', $users);
	}

	/**
	 * Mise à jour de la table {votes}.
	 *
	 * @param array $votes_v2
	 *
	 * @return bool
	 */
	private static function _updateVotes(array &$votes_v2): bool
	{
		if (!$votes_v2)
		{
			return TRUE;
		}

		if (!DB::execute("SELECT COUNT(*) FROM {votes}"))
		{
			Template::set('error', DB::getError());
			return FALSE;
		}
		if (DB::fetchVal())
		{
			Template::set('warning', 'La version 3 contient déjà des votes.');
			return FALSE;
		}

		$votes = [];
		foreach ($votes_v2 as &$i)
		{
			$votes[] =
			[
				'vote_id' => $i['vote_id'],
				'item_id' => $i['image_id'],
				'user_id' => $i['user_id'],
				'vote_rating' => $i['vote_rate'],
				'vote_date' => $i['vote_date'],
				'vote_ip' => $i['vote_ip'],
				'vote_cookie' => $i['vote_cookie']
			];
		}

		// Insertion des votes.
		if ($votes && !self::_insert('votes', $votes))
		{
			return FALSE;
		}

		return TRUE;
	}
}

class DBV2
{
	public static $PDO;
	public static $PDOStatement;
	public static $queries = [];

	private static $_allInsertId = [];
	private static $_pref = '';
	private static $_rowCount = 0;
	private static $_sql;
	private static $_time;

	public static function allInsertId(): array
	{
		return self::$_allInsertId;
	}

	public static function beginTransaction(): bool
	{
		self::_init('START TRANSACTION');

		try
		{
			self::$PDO->beginTransaction();
			self::_debug();

			return TRUE;
		}
		catch (PDOException $e)
		{
			self::_exception($e);

			return FALSE;
		}
	}

	public static function commitTransaction(): bool
	{
		self::_init('COMMIT TRANSACTION');

		try
		{
			self::$PDO->commit();
			self::_debug();

			return TRUE;
		}
		catch (PDOException $e)
		{
			self::_exception($e);

			return FALSE;
		}
	}

	public static function connect($host, $port, $name, $user, $pass, $pref): bool
	{
		try
		{
			if (self::$PDO !== NULL)
			{
				return FALSE;
			}

			self::$_pref = $pref;

			$dsn = "mysql:host=$host;port=$port;dbname=$name";
			$options = [PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES UTF8'];

			self::$PDO = new PDO($dsn, $user, $pass, $options);
			self::$PDO->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			self::$PDO->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, TRUE);

			return TRUE;
		}
		catch (PDOException $e)
		{
			self::_exception($e);
			self::disconnect();

			return FALSE;
		}
	}

	public static function disconnect(): void
	{
		self::$PDO = NULL;
	}

	public static function execute(string $sql, $params = [], array &$fetch = []): bool
	{
		$sql = Utility::trimAll($sql);
		while (substr($sql, -1) == ';')
		{
			$sql = substr($sql, 0, -1);
		}

		$sql = self::_replaceTableName($sql);

		self::_init($sql);

		try
		{
			if (strstr($sql, '"'))
			{
				throw new PDOException('Double quotes not allowed.');
			}

			else if (strstr($sql, '$'))
			{
				throw new PDOException('PHP variable not allowed.');
			}

			self::$PDOStatement = self::$PDO->prepare($sql);

			if (!is_array($params))
			{
				$params = [[$params]];
			}
			else if ((array_key_exists(0, $params) && is_array($params[0])) === FALSE)
			{
				$params = [$params];
			}

			$is_fetch = count($fetch);
			if ($is_fetch)
			{
				$fetch_method = $fetch[0];
				$fetch_arg_1 = $fetch[1] ?? NULL;
				$fetch_arg_2 = $fetch[2] ?? NULL;
				$fetch = [];
			}

			for ($i = 0; $i < count($params); $i++)
			{
				self::$PDOStatement->execute($params[$i]);

				if ($is_fetch)
				{
					$fetch[] = self::{$fetch_method}($fetch_arg_1, $fetch_arg_2);
				}

				self::$_rowCount += self::$PDOStatement->rowCount();

				self::$_allInsertId[] = self::lastInsertId();
			}

			self::_debug();

			return TRUE;
		}
		catch (PDOException $e)
		{
			self::_exception($e);

			return FALSE;
		}
	}

	public static function fetchAll(string $col1 = NULL, string $col2 = NULL): array
	{
		if ($col1 === NULL)
		{
			return self::$PDOStatement->fetchAll(PDO::FETCH_ASSOC);
		}

		$result = [];
		while ($row = self::$PDOStatement->fetch())
		{
			$result[$row[$col1]] = ($col2 !== NULL) ? $row[$col2] : $row;
		}

		return $result;
	}

	public static function fetchCol(string $col): array
	{
		return array_values(self::fetchAll($col, $col));
	}

	public static function fetchRow(): array
	{
		$row = self::$PDOStatement->fetch();
		return $row === FALSE ? [] : $row;
	}

	public static function fetchVal()
	{
		$val = self::$PDOStatement->fetch(PDO::FETCH_NUM);
		return $val === FALSE ? NULL : $val[0];
	}

	public static function getError(): string
	{
		return isset(self::$queries[count(self::$queries) - 1]['exception'])
			? self::$queries[count(self::$queries) - 1]['exception']->getMessage()
			: '';
	}

	public static function inInt(array $arr): string
	{
		return implode(', ', array_unique(array_map('intval', $arr)));
	}

	public static function inTransaction(): bool
	{
		return self::$PDO->inTransaction();
	}

	public static function lastInsertId(): int
	{
		return (int) self::$PDO->lastInsertId();
	}

	public static function likeEscape(string $str): string
	{
		return str_replace(['_', '%'], ['\_', '\%'], $str);
	}

	public static function rollbackTransaction(): bool
	{
		self::_init('ROLLBACK TRANSACTION');

		try
		{
			self::$PDO->rollback();
			self::_debug();

			return TRUE;
		}
		catch (PDOException $e)
		{
			self::_exception($e);

			return FALSE;
		}
	}

	public static function rowCount(bool $debug = FALSE): int
	{
		if (strtolower(substr(trim((string) self::$_sql), 0, 6)) == 'select')
		{
			if ($debug)
			{
				return CONF_DB_TYPE == 'sqlite' ? -1 : self::$_rowCount;
			}

			trigger_error('Do not use rowCount() after SELECT.', E_USER_NOTICE);
			return 0;
		}

		return self::$_rowCount;
	}



	private static function _debug(PDOException $e = NULL): void
	{
		self::$queries[] =
		[
			'exception' => $e,
			'file' => debug_backtrace()[2 - (int) !$e]['file'],
			'line' => debug_backtrace()[2 - (int) !$e]['line'],
			'row_count' => self::rowCount(TRUE),
			'sql' => self::$_sql,
			'time' => microtime(TRUE) - self::$_time
		];
	}

	private static function _exception(PDOException $e): void
	{
		self::_debug($e);

		ErrorHandler::dbError($e);
	}

	private static function _init(string $sql): void
	{
		self::$_allInsertId = [];
		self::$_rowCount = 0;
		self::$_sql = $sql;
		self::$_time = microtime(TRUE);
	}

	private static function _replaceTableName(string $sql): string
	{
		return preg_replace('`\{([_a-z0-9]{1,30})\}`', self::$_pref . '$1', $sql);
	}
}

?>
<!DOCTYPE html>
<html lang="<?php echo Auth::$lang; ?>">

<head>
	<title>Migration v2 vers v3 - <?php echo $tpl['app']['name']; ?></title>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width">
	<style type="text/css">
	h1,p,form,body,html
	{
		border: 0;
		margin: 0;
		padding: 0;
	}
	pre
	{
		font-family: "Courier New", serif;
		white-space: pre-wrap;
	}
	html
	{
		font-size: 100%;
	}
	body
	{
		color: black;
		font-family: Verdana, Arial, Helvetica, sans-serif;
		font-size: .8em;
		background: #fafafa;
		padding: 40px 15px;
	}
	a {
		color: black;
		text-decoration: none;
		border-bottom: 1px solid #884C2E;
		position: relative;
	}
	a:hover
	{
		border-width: 2px;
	}
	h1
	{
		font-size: 275%;
		font-weight: normal;
		border-bottom: 3px solid #8A7666;
		padding: 20px 15px 5px 30px;
		color: #555;
		font-family: Georgia, "Times New Roman", serif;
		letter-spacing: .02em;
		background: #F5F2EB url(admin/template/default/style/default/background.png);
	}
	p
	{
		margin-bottom: 15px;
	}
	strong
	{
		font-size: 110%;
	}
	#upgrade
	{
		border: 1px solid #808080;
		max-width: 650px;
		margin: 0 auto;
		text-align: left;
		background: white;
		box-shadow: 0 0 14px #989898;
	}
	#content
	{
		padding: 35px 20px 1px;
		line-height: 1.6em;
	}
	footer
	{
		text-align: center;
		border-top: 2px solid #8A7666;
		background: #F5F2EB url(admin/template/default/style/default/background.png);
		padding: 8px 0;
		margin-top: 40px;
	}
	.icon
	{
		padding: 1px 0 0 26px;
		background-repeat: no-repeat;
		background-position: 0 1px;
		min-height: 19px;
	}
	.icon span
	{
		font-weight: bold;
		color: #333;
	}
	.icon_success
	{
		background-image: url(admin/template/default/style/default/icons/18x18/success.png);
	}
	.icon_error
	{
		background-image: url(admin/template/default/style/default/icons/18x18/error.png);
	}
	.icon_warning
	{
		background-image: url(admin/template/default/style/default/icons/18x18/warning.png);
	}
	.icon_information
	{
		background-image: url(admin/template/default/style/default/icons/18x18/information.png);
	}
	input[type="submit"]
	{
		border: 1px solid silver;
		margin: 20px 0 0;
		padding: 5px 8px;
		background: linear-gradient(0deg, #E5E5E5, #FFFFFF);
		white-space: nowrap;
		border-radius: 5px;
		outline: none;
		font-size: 120%;
	}
	input[type="submit"]:active
	{
		background: linear-gradient(0deg, #FFFFFF, #E5E5E5);
	}
	input[type="submit"]:focus
	{
		border-color: gray;
	}
	input[type="submit"]:hover
	{
		box-shadow: 0 0 4px silver;
	}
	.field
	{
		font-weight: bold;
		font-size: 110%;
		color: #333;
	}
	</style>
</head>

<body>

	<div id="upgrade">
		<h1>Migration d'iGalerie 2<br>vers iGalerie 3</h1>
		<div id="content">
			<form action="" method="post">
<?php if (isset($tpl['success'])) : ?>
				<p class="icon icon_success"><span><?php echo $tpl['success']; ?></span></p>
<?php elseif (isset($tpl['error'])) : ?>
				<p class="icon icon_error"><span><?php echo $tpl['error']; ?></span></p>
<?php elseif (isset($tpl['warning'])) : ?>
				<p class="icon icon_warning"><span><?php echo $tpl['warning']; ?></span></p>
<?php else : ?>
				<p class="field icon icon_information">
					En effectuant la migration, les mots de passe de tous vos membres et de toutes vos catégories seront supprimés.
				</p>
				<p class="field icon icon_information">
					La migration peut prendre plusieurs minutes si vous avez un grand nombre d'images.
				</p>
				<div>
					<input name="anticsrf" value="<?php echo $tpl['anticsrf'](); ?>" type="hidden">
					<input name="upgrade" value="Démarrer la migration" type="submit">
				</div>
<?php endif; ?>
			</form>
		</div>
		<footer><?php printf(__('Propulsé par %s'), '<a class="ex" href="' . $tpl['app']['website'] . '">' . $tpl['app']['name'] . '</a>'); ?></footer>
	</div>

	<?php $tpl['print']('logs'); ?>

	<?php $tpl['print']('errors'); ?>

</body>

</html>